{
 "metadata": {
  "signature": "sha256:1689807736a5c04f8d6174b694da4862c47c3b4b7291a591710eadd4a683b292"
 },
 "nbformat": 3,
 "nbformat_minor": 0,
 "worksheets": [
  {
   "cells": [
    {
     "cell_type": "markdown",
     "metadata": {},
     "source": [
      "Matplotlib: interactive plotting\n",
      "======================================================================\n",
      "\n",
      "Interactive point identification\n",
      "--------------------------------\n",
      "\n",
      "I find it often quite useful to be able to identify points within a plot\n",
      "simply by clicking. This recipe provides a fairly simple\n",
      "[functor](http://en.wikipedia.org/wiki/Function_object) that can be\n",
      "connected to any plot. I've used it with both scatter and standard\n",
      "plots.\n",
      "\n",
      "Because often you'll have multiple views of a dataset spread across\n",
      "either multiple figures, or at least multiple axis, I've also provided a\n",
      "utility to link these plots together so clicking on a point in one plot\n",
      "will highlight and identify that data point on all other linked plots."
     ]
    },
    {
     "cell_type": "code",
     "collapsed": false,
     "input": [
      "import math\n",
      "\n",
      "import matplotlib.pyplot as plt\n",
      "\n",
      "class AnnoteFinder(object):\n",
      "    \"\"\"callback for matplotlib to display an annotation when points are\n",
      "    clicked on.  The point which is closest to the click and within\n",
      "    xtol and ytol is identified.\n",
      "    \n",
      "    Register this function like this:\n",
      "    \n",
      "    scatter(xdata, ydata)\n",
      "    af = AnnoteFinder(xdata, ydata, annotes)\n",
      "    connect('button_press_event', af)\n",
      "    \"\"\"\n",
      "\n",
      "    def __init__(self, xdata, ydata, annotes, ax=None, xtol=None, ytol=None):\n",
      "        self.data = list(zip(xdata, ydata, annotes))\n",
      "        if xtol is None:\n",
      "            xtol = ((max(xdata) - min(xdata))/float(len(xdata)))/2\n",
      "        if ytol is None:\n",
      "            ytol = ((max(ydata) - min(ydata))/float(len(ydata)))/2\n",
      "        self.xtol = xtol\n",
      "        self.ytol = ytol\n",
      "        if ax is None:\n",
      "            self.ax = plt.gca()\n",
      "        else:\n",
      "            self.ax = ax\n",
      "        self.drawnAnnotations = {}\n",
      "        self.links = []\n",
      "\n",
      "    def distance(self, x1, x2, y1, y2):\n",
      "        \"\"\"\n",
      "        return the distance between two points\n",
      "        \"\"\"\n",
      "        return(math.sqrt((x1 - x2)**2 + (y1 - y2)**2))\n",
      "\n",
      "    def __call__(self, event):\n",
      "\n",
      "        if event.inaxes:\n",
      "\n",
      "            clickX = event.xdata\n",
      "            clickY = event.ydata\n",
      "            if (self.ax is None) or (self.ax is event.inaxes):\n",
      "                annotes = []\n",
      "                # print(event.xdata, event.ydata)\n",
      "                for x, y, a in self.data:\n",
      "                    # print(x, y, a)\n",
      "                    if ((clickX-self.xtol < x < clickX+self.xtol) and\n",
      "                            (clickY-self.ytol < y < clickY+self.ytol)):\n",
      "                        annotes.append(\n",
      "                            (self.distance(x, clickX, y, clickY), x, y, a))\n",
      "                if annotes:\n",
      "                    annotes.sort()\n",
      "                    distance, x, y, annote = annotes[0]\n",
      "                    self.drawAnnote(event.inaxes, x, y, annote)\n",
      "                    for l in self.links:\n",
      "                        l.drawSpecificAnnote(annote)\n",
      "\n",
      "    def drawAnnote(self, ax, x, y, annote):\n",
      "        \"\"\"\n",
      "        Draw the annotation on the plot\n",
      "        \"\"\"\n",
      "        if (x, y) in self.drawnAnnotations:\n",
      "            markers = self.drawnAnnotations[(x, y)]\n",
      "            for m in markers:\n",
      "                m.set_visible(not m.get_visible())\n",
      "            self.ax.figure.canvas.draw_idle()\n",
      "        else:\n",
      "            t = ax.text(x, y, \" - %s\" % (annote),)\n",
      "            m = ax.scatter([x], [y], marker='d', c='r', zorder=100)\n",
      "            self.drawnAnnotations[(x, y)] = (t, m)\n",
      "            self.ax.figure.canvas.draw_idle()\n",
      "\n",
      "    def drawSpecificAnnote(self, annote):\n",
      "        annotesToDraw = [(x, y, a) for x, y, a in self.data if a == annote]\n",
      "        for x, y, a in annotesToDraw:\n",
      "            self.drawAnnote(self.ax, x, y, a)"
     ],
     "language": "python",
     "metadata": {},
     "outputs": [],
     "prompt_number": 1
    },
    {
     "cell_type": "markdown",
     "metadata": {},
     "source": [
      "To use this functor you can simply do something like this:"
     ]
    },
    {
     "cell_type": "code",
     "collapsed": false,
     "input": [
      "x = range(10)\n",
      "y = range(10)\n",
      "annotes = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j']\n",
      "\n",
      "fig, ax = plt.subplots()\n",
      "ax.scatter(x,y)\n",
      "af =  AnnoteFinder(x,y, annotes, ax=ax)\n",
      "fig.canvas.mpl_connect('button_press_event', af)\n",
      "plt.show()"
     ],
     "language": "python",
     "metadata": {},
     "outputs": []
    },
    {
     "cell_type": "markdown",
     "metadata": {},
     "source": [
      "This is fairly useful, but sometimes you'll have multiple views of a\n",
      "dataset and it is useful to click and identify a point in one plot and\n",
      "find it in another. The below code demonstrates this linkage and should\n",
      "work between multiple axis or figures."
     ]
    },
    {
     "cell_type": "code",
     "collapsed": false,
     "input": [
      "def linkAnnotationFinders(afs):\n",
      "  for i in range(len(afs)):\n",
      "    allButSelfAfs = afs[:i]+afs[i+1:]\n",
      "    afs[i].links.extend(allButSelfAfs)\n",
      "\n",
      "subplot(121)\n",
      "scatter(x,y)\n",
      "af1 = AnnoteFinder(x,y, annotes)\n",
      "connect('button_press_event', af1)\n",
      "\n",
      "subplot(122)\n",
      "scatter(x,y)\n",
      "af2 = AnnoteFinder(x,y, annotes)\n",
      "connect('button_press_event', af2)\n",
      "\n",
      "linkAnnotationFinders([af1, af2])"
     ],
     "language": "python",
     "metadata": {},
     "outputs": []
    },
    {
     "cell_type": "markdown",
     "metadata": {},
     "source": [
      "I find this fairly useful. By subclassing and redefining drawAnnote this\n",
      "simple framework could be used to drive a more sophisticated user\n",
      "interface.\n",
      "\n",
      "Currently this implementation is a little slow when the number of\n",
      "datapoints becomes large. I'm particularly interested in suggestions\n",
      "people might have for making this faster and better.\n",
      "\n",
      "* * * * *\n",
      "\n",
      "Handling click events while zoomed\n",
      "==================================\n",
      "\n",
      "Often, you don't want to respond to click events while zooming or\n",
      "panning (selected using the toolbar mode buttons). You can avoid\n",
      "responding to those events by checking the attribute of the toolbar\n",
      "instance. The first example below shows how to do this using the pylab\n",
      "interface."
     ]
    },
    {
     "cell_type": "code",
     "collapsed": false,
     "input": [
      "from pylab import *\n",
      "\n",
      "def click(event):\n",
      "   \"\"\"If the left mouse button is pressed: draw a little square. \"\"\"\n",
      "   tb = get_current_fig_manager().toolbar\n",
      "   if event.button==1 and event.inaxes and tb.mode == '':\n",
      "       x,y = event.xdata,event.ydata\n",
      "       plot([x],[y],'rs')\n",
      "       draw()\n",
      "\n",
      "plot((arange(100)/99.0)**3)\n",
      "gca().set_autoscale_on(False)\n",
      "connect('button_press_event',click)\n",
      "show()"
     ],
     "language": "python",
     "metadata": {},
     "outputs": []
    },
    {
     "cell_type": "markdown",
     "metadata": {},
     "source": [
      "If your application is in an wxPython window, then chances are you created a handle to the toolbar during setup, as shown in the {{{add_toolbar}}} method of the embedding_in_wx2.py example script, and can then access the {{{mode}}} attribute of that object (self.toolbar.mode in that case) in your click handling method."
     ]
    }
   ],
   "metadata": {}
  }
 ]
}